import fetch from 'node-fetch';
import fs from 'fs/promises';
import path from 'path';

interface ProviderSettings {
  openai_api_key: string;
  groq_api_key: string;
  elevenlabs_api_key: string;
  rime_ai_api_key: string;
  deepgram_api_key: string;
  twilio_account_sid: string;
  twilio_auth_token: string;
}

interface AssistantSettings {
  silence_timeout: number;
  assistant_name: string;
  first_message: string;
  system_prompt: string;
  model_provider: string;
  model: string;
  temperature: number;
  top_p: number;
  max_tokens: number;
  transcriber_provider: string;
  transcriber_model: string;
  transcriber_language: string;
  voice_provider: string;
  voice_id: string;
  voice_name?: string; // Added for ElevenLabs
  stability: number;
  clarity_boost: number;
  phone_number: string;
  forwarding_phone_number: string;
  end_call_phrases: string;
  words_to_interrupt: number;
  clear_message_delay: number;
  maximum_call_duration: number;
  response_delay: number;
  llm_request_delay: number;
  server_url: string;
  voicemail_message: string;
  end_call_message: string;
  add_voice_id_manually: boolean;
  enable_end_call_function: boolean;
  dial_keypad: boolean;
  enable_interruptions: boolean;
  hipaa_compliance: boolean;
  recording_enabled: boolean;
  provider_settings: ProviderSettings;
}

let cachedSettings: AssistantSettings[] | null = null;

export const config = {
  voice360ApiKey: process.env.VOICE360_API_KEY || '',
  assistantId: process.env.ASSISTANT_ID || '',
  perfexCrmUrl: process.env.PERFEX_CRM_URL || '',
};

const defaultSettings: AssistantSettings = {
  silence_timeout: 30,
  assistant_name: '',
  first_message: '',
  system_prompt: '',
  model_provider: 'openai',
  model: 'gpt-4o',
  temperature: 0.7,
  top_p: 1,
  max_tokens: 250,
  transcriber_provider: 'deepgram',
  transcriber_model: 'nova',
  transcriber_language: 'us',
  voice_provider: 'elevenlabs',
  voice_id: '',
  stability: 0.5,
  clarity_boost: 0.75,
  phone_number: '',
  forwarding_phone_number: '',
  end_call_phrases: '',
  words_to_interrupt: 2,
  clear_message_delay: 0,
  maximum_call_duration: 1800,
  response_delay: 0.1,
  llm_request_delay: 0.1,
  server_url: '',
  voicemail_message: '',
  end_call_message: '',
  add_voice_id_manually: false,
  enable_end_call_function: false,
  dial_keypad: false,
  enable_interruptions: false,
  hipaa_compliance: false,
  recording_enabled: false,
  provider_settings: {
    openai_api_key: '',
    groq_api_key: '',
    elevenlabs_api_key: '',
    rime_ai_api_key: '',
    deepgram_api_key: '',
    twilio_account_sid: '',
    twilio_auth_token: ''
  }
};

export async function fetchAssistantSettings(): Promise<AssistantSettings[]> {
  try {
    const url = new URL(`/admin/voice360/Api/get_all_assistant_settings`, config.perfexCrmUrl);
    
    console.log('Fetching all assistant settings from:', url.toString());
    console.log('Using API Key:', config.voice360ApiKey);

    const response = await fetch(url.toString(), {
      headers: {
        'Accept': 'application/json',
        'X-Requested-With': 'XMLHttpRequest',
        'Authorization': `Bearer ${config.voice360ApiKey}`
      }
    });

    if (!response.ok) {
      const errorText = await response.text();
      console.error('Error response body:', errorText);
      throw new Error(`Failed to fetch assistant settings. Status: ${response.status}, Error: ${errorText}`);
    }

    const data = await response.json();
    console.log('Fetched all assistant settings:', JSON.stringify(data, null, 2));

    if (!Array.isArray(data) || data.length === 0) {
      console.error('Invalid or empty data received from the API');
      return [];
    }

    const settings: AssistantSettings[] = data.map(assistantData => ({
      ...defaultSettings,
      ...assistantData,
      silence_timeout: parseInt(assistantData.silence_timeout) || defaultSettings.silence_timeout,
      temperature: parseFloat(assistantData.temperature) || defaultSettings.temperature,
      top_p: parseFloat(assistantData.top_p) || defaultSettings.top_p,
      max_tokens: parseInt(assistantData.max_tokens) || defaultSettings.max_tokens,
      stability: parseFloat(assistantData.stability) || defaultSettings.stability,
      clarity_boost: parseFloat(assistantData.clarity_boost) || defaultSettings.clarity_boost,
      words_to_interrupt: parseInt(assistantData.words_to_interrupt) || defaultSettings.words_to_interrupt,
      clear_message_delay: parseInt(assistantData.clear_message_delay) || defaultSettings.clear_message_delay,
      maximum_call_duration: parseInt(assistantData.maximum_call_duration) || defaultSettings.maximum_call_duration,
      response_delay: parseFloat(assistantData.response_delay) || defaultSettings.response_delay,
      llm_request_delay: parseFloat(assistantData.llm_request_delay) || defaultSettings.llm_request_delay,
      add_voice_id_manually: Boolean(assistantData.add_voice_id_manually),
      enable_end_call_function: Boolean(assistantData.enable_end_call_function),
      dial_keypad: Boolean(assistantData.dial_keypad),
      enable_interruptions: Boolean(assistantData.enable_interruptions),
      hipaa_compliance: Boolean(assistantData.hipaa_compliance),
      recording_enabled: Boolean(assistantData.recording_enabled),
      provider_settings: assistantData.provider_settings || {},
    }));

    // Update the assistantSettings.json file
    const cachePath = path.join(__dirname, '..', '..', 'cache_settings', 'assistantSettings.json');
    try {
      await fs.writeFile(cachePath, JSON.stringify(settings, null, 2));
      console.log('Updated assistantSettings.json:', JSON.stringify(settings, null, 2));

      // Verify the file contents after writing
      const verifyContent = await fs.readFile(cachePath, 'utf8');
      console.log('Verified contents of assistantSettings.json:', verifyContent);
    } catch (error) {
      console.error('Error writing to assistantSettings.json:', error);
    }

    cachedSettings = settings;
    console.log('All assistant settings fetched successfully');

    return settings;
  } catch (error) {
    console.error('Error fetching assistant settings:', error);
    return [];
  }
}

export function updateCachedSettings(newSettings: AssistantSettings) {
  if (cachedSettings === null) {
    cachedSettings = [newSettings];
  } else {
    // Update existing assistant or add new one
    const index = cachedSettings.findIndex(assistant => assistant.assistant_name === newSettings.assistant_name);
    if (index !== -1) {
      cachedSettings[index] = newSettings;
    } else {
      cachedSettings.push(newSettings);
    }
  }
}

export function getCachedSettings(): AssistantSettings[] | null {
  return cachedSettings;
}

export async function getAssistantSettings(phoneNumber: string): Promise<AssistantSettings | null> {
  try {
    const cachePath = path.join(__dirname, '..', '..', 'cache_settings', 'assistantSettings.json');
    const fileContent = await fs.readFile(cachePath, 'utf8');
    const assistants: AssistantSettings[] = JSON.parse(fileContent);
    const assistant = assistants.find(a => a.phone_number === phoneNumber);
    return assistant || null;
  } catch (error) {
    console.error('Error reading assistantSettings.json:', error);
    return null;
  }
}

export default config;
